/**
  *  \file action.cpp
  *  \brief Action class implementation
  *  \author Stefan Reuther
  */

#include <map>
#include "action.h"
#include "nls.h"
#include "program.h"
#include "expander.h"

static std::map<std::string, std::string> default_names;

/** Constructor.
    \param name     [in] Name of this action (standardized token)
    \param program  [in] The program we're operating for */
Action::Action(std::string name, Program& program)
    : m_Name(name), m_Program(program)
{ }

/** Destructor. */
Action::~Action()
{ }

void
Action::enumerate(KVEnumerator& e)
{
    e.handleItem("-", _("Name of this action"), m_Name);
    e.handleItem("command", _("Command to invoke"), m_CommandLine);
    e.handleItem("description", _("Description of this action"), m_Description);
}

bool
Action::assign(std::string name, std::string value, LoaderState& /*st*/)
{
    if (name == "-") {
        m_Name = value;
        return true;
    }
    if (name == "command") {
        m_CommandLine = value;
        return true;
    }
    if (name == "description") {
        m_Description = value;
        getDescription(); /* this will update the default names */
        return true;
    }
    return false;
}

/** Get preference metric. \see Program::getPreference(). */
int
Action::getPreference() const
{
    return m_Program.getPreference();
}

/** Get description of this action. A short one-line description that
    can be placed in menus. If this action has its own description,
    use that. Otherwise, uses a standard description. */
std::string
Action::getDescription()
{
    if (default_names.empty()) {
        default_names["play"] = _("Play game");
        default_names["view-vcrs"] = _("View current combat");
        default_names["view-specs"] = _("View ship and weapon specs");
        default_names["battle-sim"] = _("Simulate combat");
        default_names["maketurn"] = _("Make turn files to send to host");
        default_names["unpack"] = _("Unpack result files");
    }

    std::map<std::string,std::string>::iterator i = default_names.find(m_Name);
    if (m_Description.size()) {
        /* We have a description */
        if (i == default_names.end()) {
            /* ...but our mapping table does not yet know it */
            default_names[m_Name] = m_Description;
        }
        return m_Description;
    } else {
        /* We don't have a description */
        if (i == default_names.end()) {
            /* ...and our mapping table doesn't have one either */
            return m_Name;
        } else {
            /* Mapping table has one, use that */
            return i->second;
        }
    }
}

