/**
  *  \file game.cpp
  *  \brief Game class implementation
  *  \author Stefan Reuther
  */

#include <stdexcept>
#include "game.h"
#include "race.h"
#include "util.h"
#include "nls.h"
#include "expander.h"

/** Constructor.
    \param name Game name */
Game::Game(std::string name)
    : m_Name(name)
{ }

/** Destructor. */
Game::~Game()
{ }

void
Game::enumerate(KVEnumerator& e)
{
    e.handleItem("-",        _("Name of the game"), m_Name);
    e.handleItem("localdir", _("Local directory where this game's files are stored"), m_LocalDir);
    e.handleItem("gameurl",  _("URL of a web site for the game"), m_GameUrl);
    e.handleItem("specurl",  _("URL of specification files"), m_SpecUrl);
    e.handleItem("mapurl",   _("URL of map files"), m_MapUrl);
    e.handleItem("hostdir",  _("Host directory for locally-hosted games"), m_HostDir);
    e.handleItem("used",     _("List of used features/add-ons in this game"), m_UsedFeatures.toString());
}

bool
Game::assign(std::string name, std::string value, LoaderState& /*st*/)
{
    if (name == "-") {
        m_Name = value;
        return true;
    }
    if (name == "localdir") {
        m_LocalDir = value;
        return true;
    }
    if (name == "gameurl") {
        m_GameUrl = value;
        return true;
    }
    if (name == "specurl") {
        m_SpecUrl = value;
        return true;
    }
    if (name == "mapurl") {
        m_MapUrl = value;
        return true;
    }
    if (name == "hostdir") {
        m_HostDir = value;
        return true;
    }
    if (name == "used") {
        m_UsedFeatures.assignFromString(value);
        return true;
    }
    return false;
}

PropertyObject*
Game::createSubtree(std::string name, std::string value, LoaderState& /*st*/)
{
    if (name == "race") {
        int nr;
        if (atoi(value, nr) && nr > 0 && nr <= 11) {
            Race* r = getRaceByNumber(nr);
            if (!r) {
                r = new Race(nr);
                addRace(r);
            }
            return r;
        } else {
            fail(_("Invalid race number"));
        }
    }
    return 0;
}

/** Add a new race. */
void
Game::addRace(Race* r)
{
    safeAppend(m_Races, r);
}

/** Find race by number.
    \return race, or 0 if no such race in this game */
Race*
Game::getRaceByNumber(int nr)
{
    for (std::vector<Race*>::iterator i = m_Races.begin(); i != m_Races.end(); ++i)
        if ((*i)->getRaceNumber() == nr)
            return *i;
    return 0;
}

/** Get default race. This is the race which is selected when this
    game is selected. \todo make this configurable */
Race*
Game::getDefaultRace()
{
    if (m_Races.size())
        return m_Races.front();
    else
        return 0;
}

/** Enumerate all races. */
void
Game::enumerateRaces(ValueEnumerator<Race>& r)
{
    for (std::vector<Race*>::iterator i = m_Races.begin(); i != m_Races.end(); ++i)
        r.handleItem(**i);
}
